clear; close all; clc; warning ('on','all');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% number of chosen fundamental domain
index = 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% output data
%
% figure(1): labelled fundamental domain 
%            with the corresponding quotient structure
%
% figure(2): closure of fundamental domain mapped to the unit disc
%            with the corresponding quotient structure
%
% figure(3): dessin in the complex plane, the coordinates of the
%            zeroes, ones and poles:
%               zeros_dessin
%               ones_dessin
%               poles_dessin
%
% figure(4): (optional) symmetrized dessin in the complex plane,
%            coordinates of the zeroes, ones and poles:
%               zeros_dessin_sym 
%               ones_dessin_sym
%               poles_dessin_sym
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% load Magma-computed fundamental domains, gluing data from file
clear('matlab_input.m')
run('matlab_input.m')

degree = size(permx,2);
fprintf('degree: %d\n',degree);
fprintf('number of computed fundamental domains: %d (choose index between 1 and %d)\n',size(FDList,2),size(FDList,2));

FD = FDList{index};
GluingData = GluingDataList{index};

%% Computing a fundamental domain
% Create hyperbolic kite in H
[z1,z2,z3,cz1,cz2] = CreateHyperbolicKite(a,b,c);

% define hyperbolic rotation matrices DA and DB
lambda = (cos(pi/a) * cos(pi/b) + cos(pi/c))/(sin(pi/a)*sin(pi/b));
mu     = lambda + sqrt(lambda^2 - 1);
DA = [cos(pi/a) sin(pi/a); -sin(pi/a) cos(pi/a)];
DB = [cos(pi/b) mu*sin(pi/b); -(sin(pi/b))/mu cos(pi/b)];

% Assemble transformation matrices (products of DA and DB) 
% according to the chosen fundamental domain
[m,n] = size(FD);
T = cell(1,m);
for j=1:m
    T{j} = eye(size(DA));
    s = 1;
    for h=1:(n/2) 
        T{j} = T{j}*DA^FD(j,s);
        T{j} = T{j}*DB^FD(j,s+1);
        s = s+2;
    end
end

% Apply transformation matrices to the hyperbolic kite to 
% obtain the fundamental domain
moz1 = cell(1,m); moz2 = cell(1,m); cmoz1 = cell(1,m); cmoz2 = cell(1,m); cz3 = cell(1,m);
for j=1:m
   moz1{j}  = w(mo(T{j},z1));  
   moz2{j}  = w(mo(T{j},z2));  
   cmoz1{j} = w(mo(T{j},cz1));
   cmoz2{j} = w(mo(T{j},cz2));
   cz3{j}   = w(mo(T{j},z3));
end

% plot the labelled fundamental domain
figure(1)
clf
title('labelled fundamental domain')
hold on
axis equal
for j=1:m
  plot(real(moz1{j}),imag(moz1{j}) ,'black', 'LineWidth',.15)
  plot(real(moz2{j}),imag(moz2{j}), 'black','LineWidth',.15) 
  plot(real(cmoz1{j}),imag(cmoz1{j}) ,'black','LineWidth',.15)
  plot(real(cmoz2{j}),imag(cmoz2{j}) , 'black','LineWidth',.15) 
  plot(real(cz3{j}),imag(cz3{j}) , 'red','LineWidth',1.5)
  text(real(cz3{j}(100)),imag(cz3{j}(100)),num2str(j))
end
plot(cos(0:0.01:2*pi),sin(0:0.01:2*pi),'k')


%% mapping the fundamental domain to the unit disc

dessin_edges = zeros(m,size(z3,2));
for j = 1:m
    dessin_edges(j,:) = cz3{j};
end

matcmoz1 = cell2mat(cmoz1');
dessin_poles = matcmoz1(:,end);

border_edges = zeros(size(GluingData,1),size(z1,2)+1);
for j = 1:size(GluingData,1)
    edge = GluingData(j,:);
    if edge(3) == 0 && edge(4) == 1
        border_edge = cz2;
    end
    if edge(3) == 0 && edge(4) == -1
        border_edge = z2;
    end
    if edge(3) == 1 && edge(4) == 1
        border_edge = cz1;
    end
    if edge(3) == 1 && edge(4) == -1
        border_edge = z1;
    end
    border_edges(j,:) = [edge(5),w(mo(T{edge(1)},border_edge))];
end

[polygon_vertices, L] = CreatePolygonAndGluingData(border_edges);
poly = polygon(fliplr(polygon_vertices.'));

% plot quotient structure on the border of the fundamental domain 
pv = polygon_vertices;
%plot(real(pv),imag(pv),'x','color',[0 0.3 0]);
q = size(pv,1);
for j = 1: size(L,1)
    for jj = [1,2]
    p1 = pv(L(j,jj));
    p2 = pv(mod(L(j,jj)-2,q)+1);
    m=(p1+p2)/2;
    text(real(m),imag(m),num2str(j),'color',[0 0.3 0])
    end
end

fprintf('computation of SC map for a polygon with %d edges starts...\n',length(poly)) 
sc_map = diskmap(poly);
fprintf('accuracy of computed SC map: %d\n',accuracy(sc_map))

fprintf('mapping fundamental domain and dessin to D...\n')
polygon_vertices = evalinv(sc_map,polygon_vertices);
dessin_edges = evalinv(sc_map,dessin_edges);
dessin_poles = evalinv(sc_map,dessin_poles);

%% plot the closure of fundamental domain mapped to the  
%% unit disc with the corresponding quotient structure 
figure(2)
clf
title('fundamental domain mapped to the unit disc')
hold on
axis equal
% plot circle
plot(cos(0:0.01:2*pi),sin(0:0.01:2*pi),'Color','k')
% plot dessin
for j=1:size(dessin_edges,1)
    plot(real(dessin_edges(j,:)),imag(dessin_edges(j,:)))
    text(real(dessin_edges(j,end/2)),imag(dessin_edges(j,end/2)),num2str(j))
end
% plot poles
plot(real(dessin_poles(:,1)),imag(dessin_poles(:,1)),'bx')
% plot border of the fundamental domain
pv = polygon_vertices;
plot(real(pv),imag(pv),'x','color',[0 0.3 0]);
q = size(pv,1);
for j = 1:size(L,1)
    for jj = [1,2]
    p1 = pv(L(j,jj));
    p2 = pv(mod(L(j,jj)-2,q)+1);
    m = (p1+p2)/2;
    m = m/norm(m);
    text(real(m),imag(m),num2str(j),'color',[0 0.3 0])
    end
end

%% Welding process
disp('start welding')
[zz,poles] = Welding(polygon_vertices,dessin_edges,dessin_poles,L);
disp('welding finished')

%% Symmetrizing process (if possible)
if size(ConjugateEdges,2)>0
    real_edges = find(~(ConjugateEdges - (1:degree)))';
    nonreal_edges = find((ConjugateEdges - (1:degree)))';
    edges = [real_edges; nonreal_edges];
    if size(real_edges,1) > 0 
        vektor = [edges(1),edges(1),edges(2)];
        r1 = vektor(1); r2 = ConjugateEdges(r1);
        A = (zz(r1,1)+zz(r2,1))/2;   
        r1 = vektor(2); r2 = ConjugateEdges(r1);
        B = (zz(r1,end)+zz(r2,end))/2;  
        r1 = vektor(3); r2 = ConjugateEdges(r1);
        C = (zz(r1,100)+zz(r2,100))/2; 
        zz = moeb(zz,A,B,C,-1,0,1);
        poles = moeb(poles,A,B,C,-1,0,1);       
    else
        r1 = edges(1); r2 = ConjugateEdges(r1);
        A = zz(r1,100);
        B = zz(r2,100);   
        r1 = edges(end); r2 = ConjugateEdges(r1);
        C = (zz(r1,end)+zz(r2,end))/2;  
        zz = moeb(zz,A,B,C,1i,-1i,1);
        poles = moeb(poles,A,B,C,1i,-1i,1);  
    end
end

%% computing the poles
poles2 = poles;
for j = 1:size(permz_cycles,2)
    c = cell2mat(permz_cycles(j));
    for k = cell2mat(permz_cycles(j))
        poles2(k) = mean(poles(c));
    end 
end
poles = poles2;
polesneu = zeros(size(permz_cycles,2),2);
for j = 1:size(permz_cycles,2)
     c = cell2mat(permz_cycles(j));
     polesneu(j,:) = [mean(poles(c)),size(c,2)];
end

%% plot the resulting dessin in the complex plane
figure(3);
clf
title('dessin')
hold on
axis equal

% plot dessin 
for j=1:size(zz,1)
    plot(real(zz(j,:)),imag(zz(j,:)))
    text(real(zz(j,end/2)),imag(zz(j,end/2)),num2str(j))
end

% plot poles
plot(real(polesneu(:,1)),imag(polesneu(:,1)),'bx')

% output data
zeros_dessin = zz(:,1);
ones_dessin = zz(:,end);
poles_dessin = poles;

%% Smoothing process (if possible), i.e. averaging out complex conjugate points of the dessin
if size(ConjugateEdges,2) > 0 
    zz_smooth = zeros(size(zz));    
    for count = 1:degree
        A = zz(count,:); B = zz(ConjugateEdges(count),:);
        zz_smooth(count,:) = real(1/2 * (A + B)) + 1i*(imag(1/2 * (A - B)));  
    end
    LC = 1:size(polesneu,1);
    poles2 = zeros(size(poles));
    for j = 1:size(polesneu,1)        
        if ismember(j,LC)  
            c = cell2mat(permz_cycles(j));
            tt = ConjugateEdges(permx(c(1)));          
            for jj=1:size(permz_cycles,2)
                c = cell2mat(permz_cycles(jj));
                f = find(c==tt);              
                if size(f,2) > 0
                    break
                end      
            end             
            if jj == j 
                poles2(cell2mat(permz_cycles(j))) = real(polesneu(j,1));
            else
                A = polesneu(j,1); B = polesneu(jj,1);
                rr =  real(1/2 * (A + B)) + 1i*(imag(1/2 * (A - B)));
                poles2(cell2mat(permz_cycles(j))) = rr;
                poles2(cell2mat(permz_cycles(jj))) = conj(rr);
            end               
            LC = setdiff(LC,[j,jj]);     
        end
    end
    poles3 = zeros(size(permz_cycles,2),2);
    for j = 1:size(permz_cycles,2)
        c = cell2mat(permz_cycles(j));
        poles3(j,:) = [mean(poles2(c)),size(c,2)];
    end

%% plot the resulting symmetrized dessin in the complex plane (if possible) 
    figure(4);
    clf
    title('symmetrized dessin')
    hold on
    axis equal
    % plot real dessin
    for j=1:size(zz_smooth,1)
        plot(real(zz_smooth(j,:)),imag(zz_smooth(j,:)))
        text(real(zz_smooth(j,end/2)),imag(zz_smooth(j,end/2)),num2str(j))
    end
    % plot poles
    for j=1:size(poles3,1)
        plot(real(poles3(j,1)),+imag(poles3(j,1)),'bx')
    end
    % output data
    zeros_dessin_sym = zz_smooth(:,1);
    ones_dessin_sym = zz_smooth(:,end);
    poles_dessin_sym = poles2;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function res = mo(M,z)
% computes the image of z under the Moebius transformation given by a 2x2
% matrix M
    res = (M(1,1)*z + M(1,2)) ./ (M(2,1)*z + M(2,2));
end

function res = w(z)
% maps the upper half plane to the unit disc via z -> (z-i)/(z+i)
    res = (z-1i) ./ (z+1i);
end

function res = moeb(z,z1,z2,z3,w1,w2,w3)
% computes the image of z of a Moebius transformation mapping [z1,z2,z3] to [w1,w2,w3] 
res = (w1*(-w2*(z1 - z2)*(z - z3*ones(size(z))) + w3*(z - z2*ones(size(z)))*(z1 - z3)) - ...
  w2* w3* (z - z1*ones(size(z)))* (z2 - z3))./(w3* (z1 - z2)* (z - z3*ones(size(z))) - ...
  w2* (z - z2*ones(size(z)))* (z1 - z3) + w1* (z - z1*ones(size(z)))* (z2 - z3));
end